/*
   Project name:
     Arduino Ground Balance Test
   Copyright:
     (c) George Overton, 2021.
   Revision History:
     - V1.0 Initial release
   Description:
     GB is based on a technique from the paper:
     UXO Target Detection and Discrimination with Electromagnetic Differential Illumination
     k = T2 x t + T1        m1 = s + g        m2 = s + kg        s = km1 - m2
         --   ------                                                 --------
         T1   t + T2                                                   k - 1
   NOTES:
     TX OSC signal must be present for +5V supply to work.
     Mosfet switching levels assume active turn-off mod has been applied
*/

// Pin assignments
// D0 and D1 are used by hardware UART
byte txPin = 8;                         // Assign TX output to D8
byte s1Pin = 9;                         // Assign sample 1 pulse to D9
byte s2Pin = 10;                        // Assign sample 2 pulse to D10
byte audioPin = 11;                     // Assign audio chopper to D11
byte gbSwPin = 12;                      // Assign GB switch to D12
// D13 is attached to an onboard LED
// Use only as an output
// Requires external pull-down resistor if used as an input
byte potPin = A0;                       // Assign pot to A0
// A6 and A7 can only be used as inputs

// Program constants
// Initial detector timings are in micro-seconds
const unsigned int initTxGbOff = 100;    // TX-on (GB off)
const unsigned int initTxNarrow = 20;    // TX-on (Narrow pulse)
const unsigned int initTxWide = 200;     // TX-on (Wide pulse)
const unsigned int initTxPeriod = 1000;  // TX period
const unsigned int initTgd = 17;         // Initial target delay relative to TX end
const unsigned int initTgwGbOff = 50;    // Initial target width relative to TGD end (GB off)
const unsigned int initTgwNarrow = 20;   // Initial target width relative to TGD end (Narrow)
const unsigned int initTgwWide = 20;     // Initial target width relative to TGD end (Wide)
const unsigned int initEfd = 500;        // Initial EF delay relative to TX end
const byte mosfetOn = LOW;               // Mosfet turns on when transmitter input high
const byte mosfetOff = HIGH;             // Mosfet turns off when transmitter input low
const byte syncDemodOn = LOW;            // Sample gate on pulse
const byte syncDemodOff = HIGH;          // Sample gate off pulse
const byte gbOff = HIGH;                 // GB off when switch input high
const byte gbOn = LOW;                   // GB on when switch input low

// Detector timings
unsigned int txGbOff = initTxGbOff;      // TX pulse (GB off)
unsigned int txNarrow = initTxNarrow;    // TX pulse (Narrow)
unsigned int txWide = initTxWide;        // TX pulse (Wide)
unsigned int txPeriod = initTxPeriod;    // TX period
unsigned int tgd = initTgd;              // Target sample delay
unsigned int tgwGbOff = initTgwGbOff;    // Target sample width (GB off)
unsigned int tgwNarrow = initTgwNarrow;  // Target sample width (Narrow)
unsigned int tgwWide = initTgwWide;      // Target sample width (Wide)
unsigned int efd = initEfd;              // EF sample delay
unsigned int efwGbOff = tgwGbOff;        // EF sample width (GB off)

// Timing offsets
// GB off
unsigned int txOffsetGbOff = 4;          // TX pulse offset
unsigned int txPeriodOffsetGbOff = 110;  // TX period offset
unsigned int tgdOffsetGbOff = 5;         // TGD offset
unsigned int tgwOffsetGbOff = 6;         // TGW offset
unsigned int efdOffsetGbOff = 8;         // EFD offset
unsigned int efwOffsetGbOff = 7;         // EFW offset
// Narrow pulse
unsigned int txOffsetNarrow = 4;         // TX pulse offset
unsigned int txPeriodOffsetNarrow = 10;  // TX period offset
unsigned int tgdOffsetNarrow = 5;        // TGD offset
unsigned int tgwOffsetNarrow = 6;        // TGW offset

// Wide pulse
unsigned int txOffsetWide = 4;           // TX pulse offset
unsigned int txPeriodOffsetWide = 120;   // TX period offset
unsigned int tgdOffsetWide = 5;          // TGD offset
unsigned int tgwOffsetWide = 6;          // TGW offset

// Program variables
unsigned int txCountGbOff;         // TX pulse (GB off)
unsigned int txCountNarrow;        // TX pulse (Narrow)
unsigned int txCountWide;          // TX pulse (Wide)
unsigned int txPeriodCountGbOff;   // TX period (GB off)
unsigned int txPeriodCountNarrow;  // TX period (Narrow)
unsigned int txPeriodCountWide;    // TX period (Wide)
unsigned int tgdCountGbOff;        // Target delay count (GB off)
unsigned int tgdCountNarrow;       // Target delay count (Narrow)
unsigned int tgdCountWide;         // Target delay count (Wide)
unsigned int tgwCountGbOff;        // Target width count (GB off)
unsigned int tgwCountNarrow;       // Target width count (Narrow)
unsigned int tgwCountWide;         // Target width count (Wide)
unsigned int efdCountGbOff;        // EF delay count (GB off)
unsigned int efwCountGbOff;        // EF width count (GB off)
word potVal = 0;                   // Pot value
byte gbSw;                         // GB switch position (on or off)
byte gbState;                      // Current state of GB (on or off)

void setup() {
  pinMode(txPin, OUTPUT);             // Set TX pin to output mode
  pinMode(s1Pin, OUTPUT);             // Set S1 pin to output mode
  pinMode(s2Pin, OUTPUT);             // Set S2 pin to output mode
  pinMode(gbSwPin, INPUT_PULLUP);     // Set GB switch pin to input mode with pullup resistor
  gbState = gbOff;                    // Set GB switch state to off
  calcTimerValuesGbOff();             // Calculate timer values when GB is off
  noInterrupts();                     // Disable interrupts
  digitalWrite(txPin, mosfetOff);     // Turn off Mosfet
  digitalWrite(s1Pin, syncDemodOff);  // Turn off S1 gate
  digitalWrite(s2Pin, syncDemodOff);  // Turn off S2 gate
  analogWrite(audioPin, 127);         // Set audioPin with 50% duty cycle PWM
}

// Calculate timer values when GB is off
void calcTimerValuesGbOff() {
  txCountGbOff = txGbOff - txOffsetGbOff; 
  tgdCountGbOff = tgd - tgdOffsetGbOff;
  tgwCountGbOff = tgwGbOff - tgwOffsetGbOff;
  efdCountGbOff = efd - tgd - tgwGbOff - efdOffsetGbOff;
  efwCountGbOff = tgwGbOff - efwOffsetGbOff;
  txPeriodCountGbOff = txPeriod - efd - tgwGbOff - txPeriodOffsetGbOff;
}

// Calculate timer values when GB is on
void calcTimerValuesGbOn() {
  txCountNarrow = txNarrow - txOffsetNarrow; 
  txCountWide = txWide - txOffsetWide;
  tgdCountNarrow = tgd - tgdOffsetNarrow;
  tgdCountWide = tgd - tgdOffsetWide;
  tgwCountNarrow = tgwNarrow - tgwOffsetNarrow;
  tgwCountWide = tgwWide -tgwOffsetWide;
  txPeriodCountNarrow = txPeriod - txNarrow - tgd - tgwNarrow - txPeriodOffsetNarrow;
  txPeriodCountWide = txPeriod - txWide - tgd - tgwWide - txPeriodOffsetWide;
}

void loop() {
  if (gbState == gbOff) {
    
    digitalWrite(txPin, mosfetOn);           // Turn on mosfet
    delayMicroseconds(txCountGbOff);         // TX pulse (GB off)
    
    digitalWrite(txPin, mosfetOff);          // Turn off mosfet
    delayMicroseconds(tgdCountGbOff);        // Target sample delay
    
    digitalWrite(s1Pin, syncDemodOn);        // Turn on sample gate 1
    delayMicroseconds(tgwCountGbOff);        // Target sample width (GB off)
    
    digitalWrite(s1Pin, syncDemodOff);       // Turn off sample gate 1
    delayMicroseconds(efdCountGbOff);        // EF sample delay (GB off)
    
    digitalWrite(s2Pin, syncDemodOn);        // Turn on sample gate 2
    delayMicroseconds(efwCountGbOff);        // EF sample width (GB off)
    
    digitalWrite(s2Pin, syncDemodOff);       // Turn off sample gate 2
    delayMicroseconds(txPeriodCountGbOff);   // TX period delay (GB off)
    
  } else {
    
    digitalWrite(txPin, mosfetOn);           // Turn on mosfet
    delayMicroseconds(txCountNarrow);        // TX pulse (Narrow)
    
    digitalWrite(txPin, mosfetOff);          // Turn off mosfet
    delayMicroseconds(tgdCountNarrow);       // Target sample delay
    
    digitalWrite(s1Pin, syncDemodOn);        // Turn on sample gate 1
    delayMicroseconds(tgwCountNarrow);       // Target sample width (Narrow)

    digitalWrite(s1Pin, syncDemodOff);       // Turn off sample gate 1
    delayMicroseconds(txPeriodCountNarrow);  // TX period delay (Narrow)

    digitalWrite(txPin, mosfetOn);           // Turn on mosfet
    delayMicroseconds(txCountWide);          // TX pulse (Wide)
    
    digitalWrite(txPin, mosfetOff);          // Turn off mosfet
    delayMicroseconds(tgdCountWide);         // Target sample delay
    
    digitalWrite(s2Pin, syncDemodOn);        // Turn on sample gate 2
    delayMicroseconds(tgwCountWide);         // Target sample width (Wide)

    digitalWrite(s2Pin, syncDemodOff);       // Turn off sample gate 2
    delayMicroseconds(txPeriodCountWide);    // TX period delay (Wide)

    potVal = analogRead(potPin);             // Read pot value
    tgwNarrow = initTgwNarrow + potVal / 2;  // Adjust GB sample width
    calcTimerValuesGbOn();                   // Calculate new timer values for GB on
  }

  gbSw = digitalRead(gbSwPin);               // Get GB switch position
  if (gbState != gbSw) {                     // Check if current GB state has changed
    if (gbSw == gbOff) {                     // Check if state changed
      calcTimerValuesGbOff();
    } else {
      calcTimerValuesGbOn();
    }
    gbState = gbSw;                          // Save GB switch state
  }
}
